/* VIDEO.I  Contains ROM BIOS calls to be used with Turbo C */

#define  ROM  0x10

union  REGS  inreg, outreg;

int videomode (int *ncols)             /* get current display mode */
{                              /* return number of cols via *ncols */
  inreg.h.ah = 0x0F;
  int86 (ROM, &inreg, &outreg);
  *ncols = outreg.h.ah;                          /* number of cols */
  return (outreg.h.al);                             /* return mode */
} /*----------------------------- */

int activepage (void)                /* return active display page */
{
  inreg.h.ah = 0x0F;
  int86 (ROM, &inreg, &outreg);
  return (outreg.h.bh);
} /*----------------------------- */

void setmode (int mode)                          /* set video mode */
{
  inreg.h.al = mode;
  inreg.h.ah = 0x00;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void setcursor (int start, int end)            /* set cursor shape */
{
  inreg.h.ch = start;
  inreg.h.cl = end;
  inreg.h.ah = 0x01;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

int curstart (void)                    /* get cursor starting line */
{
  inreg.h.bh = 0;                /* cursor same shape in all pages */
  inreg.h.ah = 0x03;
  int86 (ROM, &inreg, &outreg);
  return (outreg.h.ch);
} /* ----------------------------- */

int cursend (void)                       /* get cursor ending line */
{
  inreg.h.bh = 0;
  inreg.h.ah = 0x03;
  int86 (ROM, &inreg, &outreg);
  return (outreg.h.cl);
} /* ----------------------------- */

void cursoff (void)                             /* turn cursor off */
{
  inreg.h.ch = curstart () | 0x10;                /* turn on bit 4 */
  inreg.h.cl = cursend ();
  inreg.h.ah = 0x01;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void curson (void)                               /* turn cursor on */
{
  inreg.h.ch = curstart () & 0x07;     /* turn off high order bits */
  inreg.h.cl = cursend ();
  inreg.h.ah = 0x01;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void gotoxy (int col, int row, int page)         /* set cursor pos */
{                                             /* must specify page */
  inreg.h.bh = page;
  inreg.h.dh = row;
  inreg.h.dl = col;
  inreg.h.ah = 0x02;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

int wherex (int page)              /* return cursor column in page */
{
  inreg.h.bh = page;
  inreg.h.ah = 0x03;
  int86 (ROM, &inreg, &outreg);
  return (outreg.h.dl);
} /* ----------------------------- */

int wherey (int page)                 /* return cursor row in page */
{
  inreg.h.bh = page;
  inreg.h.ah = 0x03;
  int86 (ROM, &inreg, &outreg);
  return (outreg.h.dh);
} /* ----------------------------- */

void setpage (int page)                 /* set active display page */
{
  inreg.h.al = page;
  inreg.h.ah = 0x05;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void cls (void)                             /* clear active screen */
{
  inreg.h.al = 25;                                /* entire screen */
  inreg.h.bh = 0x07;                       /* set to grey on black */
  inreg.h.ah = 0x06;
  inreg.h.cl =  0; inreg.h.ch =  0;
  inreg.h.dl = 79; inreg.h.dh = 24;
  int86 (ROM, &inreg, &outreg);
  gotoxy (0, 0, activepage ());
} /* ----------------------------- */

void window (int x1, int y1,           /* window upper left corner */
             int x2, int y2,                 /* lower right corner */
             char attrib)                 /* text attribute inside */
{
  inreg.h.al = y2 - y1 + 1;                 /* clear entire window */
  inreg.h.bh = attrib;
  inreg.h.cl = x1; inreg.h.ch = y1;
  inreg.h.dl = x2; inreg.h.dh = y2;
  inreg.h.ah = 0x06;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void winscroll (int x1, int y1, int x2, int y2,   /* scroll window */
                int attr)                       /* one line upward */
{
  inreg.h.al = 1;
  inreg.h.cl = x1; inreg.h.ch = y1;
  inreg.h.dl = x2; inreg.h.dh = y2;
  inreg.h.bh = attr;
  inreg.h.ah = 0x06;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

char chattr (int foregrnd, int backgrnd)    /* character attribute */
{
  return ((backgrnd << 4) + foregrnd);
} /* ----------------------------- */

char rdchara (int page,                   /* read char at curs pos */
              char *attrib)         /* return attribute indirectly */	
{
  inreg.h.bh = page;
  inreg.h.ah = 0x08;
  int86 (ROM, &inreg, &outreg);
  *attrib = outreg.h.ah;
  return (outreg.h.al);
} /* ----------------------------- */

void wrtcha (char ch, char attrib,          /* write char + attrib */
             int page)                    /* at cursor pos on page */
{                                /* NOTE: does note advance cursor */
  inreg.h.al = ch;
  inreg.h.bh = page;
  inreg.h.bl = attrib;
  inreg.x.cx = 1;
  inreg.h.ah = 0x09;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void wrtstra (char *str,                           /* write string */
              char attrib,                       /* with attribute */
              int page)                                 /* to page */
{                                   /* starting at cursor position */
int	c, r, n, p = 0;

  videomode (&n);                           /* get width of screen */
  r = wherey (page);                            /* get current row */
  while (str[p]) {
    wrtcha (str[p++],attrib,page);              /* write next char */
    if ((c = wherex (page)) < (n-1))
      gotoxy (c + 1, r, page);                   /* advance cursor */
    else
      gotoxy (0, ++r, page);                          /* else wrap */
  }
} /* ----------------------------- */

void wrtch (char ch, int color,             /* write char in color */
            int page)                /* at cursor position on page */
{
  inreg.h.al = ch;
  inreg.h.bl = color;
  inreg.h.bh = page;
  inreg.x.cx = 1;
  inreg.h.ah = 0x0A;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void wrtstr (char *str,                            /* write string */
             char color,                               /* in color */
             int page)              /* starting at cursor position */
{
int 	c, r, n, p = 0;

  videomode (&n);                           /* get width of screen */
  r = wherey (page);                            /* get current row */
  while (str[p]) {
    wrtcha (str[p++], color, page);             /* write next char */
    if ((c = wherex (page)) < (n-1))
      gotoxy (c + 1, r, page);                   /* advance cursor */
    else
      gotoxy (0, ++r, page);                          /* else wrap */
  }
} /* ----------------------------- */

void palette (int palno)                      /* set color palette */
{                                   /* valid only in mode 4 on CGA */
  inreg.h.bh = 1;
  inreg.h.bl = palno;
  inreg.h.ah = 0x0B;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void graphbackround (int color)          /* set graphics b/g color */
{
  inreg.h.bh = 0;
  inreg.h.bl = color;
  inreg.h.ah = 0x0B;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

void plot (int x, int y, int pixel)           /* plot pixel at x, y */
{
  inreg.h.al = pixel;                       /* pixel (color) value */
  inreg.h.bh = 0;
  inreg.x.cx = x;
  inreg.x.dx = y;
  inreg.h.ah = 0x0C;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

int pixel (int x, int y)             /* return pixel value at x, y */
{
  inreg.x.cx = x;
  inreg.x.dx = y;
  inreg.h.ah = 0x0D;
  int86 (ROM, &inreg, &outreg);
} /* ----------------------------- */

